package {
	//for FLARToolkit 2.5.3
	
	import flash.display.Bitmap;
	import flash.display.Shape;
	import flash.display.Sprite;
	import flash.events.Event;
	
	import jp.nyatla.nyartoolkit.as3.core.types.NyARIntSize;
	import org.libspark.flartoolkit.core.FLARMat;
	import org.libspark.flartoolkit.core.param.FLARParam;
	import org.libspark.flartoolkit.core.types.FLARIntSize;
	import org.libspark.flartoolkit.core.analyzer.raster.threshold.FLARRasterThresholdAnalyzer_SlidePTile;
	import org.libspark.flartoolkit.core.transmat.FLARTransMatResult;
	
	import frocessing.core.F5Graphics3D;
	
	public class F5ARApp extends ARAppBase {
		
		protected var _base:Sprite;		
		
		//ar
		protected var _resultMat:FLARTransMatResult = new FLARTransMatResult();
		private var _threshold:int = 110;
		private var _threshold_detect:FLARRasterThresholdAnalyzer_SlidePTile;
		protected var _is_raster_view:Boolean = false;
		
		//view
		protected var _view:Shape;
		protected var _fg:F5Graphics3D;
		
		
		public function F5ARApp() {
		}
		
		protected override function init(cameraFile:String, codeFile:String, canvasWidth:int = 320, canvasHeight:int = 240, codeWidth:int = 80):void {
			addEventListener(Event.INIT, _onInit, false, int.MAX_VALUE);
			super.init(cameraFile, codeFile, canvasWidth, canvasHeight, codeWidth);
		}
		
		private function _onInit(e:Event):void {
			_base = addChild(new Sprite()) as Sprite;
			
			//
			_capture.width = 640;
			_capture.height = 480;
			_base.addChild(_capture);
			
			//
			this._view = _base.addChild( new Shape() ) as Shape;
			this._fg = new F5Graphics3D( this._view.graphics, 640, 480 );
			this._view.x = -4; // 4pix ??? 
			
			//
			this._threshold_detect = new FLARRasterThresholdAnalyzer_SlidePTile(15, 4);
			if (this._is_raster_view) {
				var binRasterBitmap:Bitmap = new Bitmap(this._detector.thresholdedBitmapData);
				_base.addChild(binRasterBitmap);
			}
			
			//
			_initCamera( this._param );
			
			setup3d( _fg );
			
			this.addEventListener(Event.ENTER_FRAME, this._onEnterFrame);
		}
		
		protected function setup3d( fg:F5Graphics3D ):void
		{
			;
		}
		
		protected function draw3d( fg:F5Graphics3D ):void
		{
			;
		}
		
		private function _onEnterFrame(e:Event = null):void {
			_capture.bitmapData.draw(_video);
			var detected:Boolean = false;
			try {
				detected = _detector.detectMarkerLite(_raster, _threshold) && _detector.getConfidence() > 0.5;
			} catch (e:Error) {}
			
			if (detected) {
				_detector.getTransformMatrix(_resultMat);
				_fg.beginDraw();
				_fg.translate( _fg.width * 0.5, _fg.height * 0.5 );
				_fg.applyMatrix(  _resultMat.m00,  _resultMat.m10, -_resultMat.m20,
								 -_resultMat.m01, -_resultMat.m11,  _resultMat.m21,
								  _resultMat.m02,  _resultMat.m12, -_resultMat.m22,
								  _resultMat.m03,  _resultMat.m13, -_resultMat.m23 );
				//_fg.scale( 1, -1, 1 );
				draw3d( _fg );
				_fg.endDraw();
				_view.visible = true;
			} else {
				_view.visible = false;
				// マーカがなければ、探索+DualPTailで基準輝度検索
				// マーカーが見つからない場合、処理が重くなるので状況に応じてコメントアウトすると良い
				var th:int=this._threshold_detect.analyzeRaster(_raster);
				this._threshold=(this._threshold+th)/2;
				// trace("[threshold] : " + this._threshold);
			}
		}
		
		public function set mirror(value:Boolean):void {
			if (value) {
				this._base.scaleX = -1;
				this._base.x = 640;
			} else {
				this._base.scaleX = 1;
				this._base.x = 0;
			}
		}
		
		public function get mirror():Boolean {
			return this._base.scaleX < 0;
		}
		
		private static const NEAR_CLIP:Number = 10;
		private static const FAR_CLIP:Number = 10000;
		
		private function _initCamera( param:FLARParam ):void
		{			
			const size:NyARIntSize = param.getScreenSize();
			const width:int  = size.w;
			const height:int = size.h;
			const tMat:FLARMat = new FLARMat (3, 4);
			const iMat:FLARMat = new FLARMat (3, 4);
			param.getPerspectiveProjectionMatrix().decompMat(iMat, tMat);
			var icpara:Vector.<Vector.<Number>> = iMat.getArray();
			for (var i:int = 0; i < 4; i++) {
				icpara[1][i] = (height - 1) * (icpara[2][i]) - icpara[1][i];
			}
			_fg.camera( _fg.width / 2, _fg.height / 2, 0, _fg.width / 2, _fg.height / 2, -1, 0, 1, 0 );
			var top:Number    = -NEAR_CLIP * 0.5 * height / icpara[1][1] / icpara[2][2];
			var bottom:Number = -top;
			var left:Number   = -NEAR_CLIP * 0.5 * width / icpara[0][0] / icpara[2][2];
			var right:Number  = -left;
			_fg.frustum( left, right, bottom, top, NEAR_CLIP, FAR_CLIP );
		}
	}
}